#!/usr/bin/python

import json

from .collect import collectjson
from .setversion import setversion
from .appcache import appcache
from .compilers import optimize, scss
from .init import init


class Builder(object):
    conf = None
    indir = None
    outdir = None
    version = ""

    def __init__(self, version=None, config="app.build.json"):
        # Load configuration file
        self.conf = json.load(open(config))
        if 'optimize' not in self.conf:
            raise Exception("No optimize section in conf file!")

        # Determine input and output directories
        self.indir = self.conf['optimize']['appDir']
        self.outdir = self.conf['optimize']['dir']
        if version is not None:
            self.version = version

    def init(self, conf=None):
        if conf is None:
            conf = self.conf.get('init', {})
        init(conf, self.indir)

    def build(self):
        self.init()

        # Save version information
        if 'setversion' in self.conf or self.version != "":
            self.setversion()

        # Collect files into JSON(P) objects
        if 'collectjson' in self.conf:
            self.collectjson()

        if 'scss' in self.conf:
            self.scss()

        # Compile Javascript / CSS (using r.js)
        self.optimize()

        # Generate HTML5 Cache manifests
        if 'appcache' in self.conf:
            self.appcache()

    def setversion(self, directory=None, conf=None):
        if directory is None:
            directory = self.indir
        if conf is None:
            conf = self.conf.get('setversion', {})
        if self.version != '':
            conf['version'] = self.version
        self.version = setversion(conf, directory)

    def collectjson(self, directory=None, conf=None):
        """Collect files into JSON dictionaries"""
        if directory is None:
            directory = self.indir
        if conf is None:
            conf = self.conf.get('collectjson', {})

        if isinstance(conf, dict):
            collectjson(conf, directory)
        else:
            # Assume multiple collectjson configurations
            for c in conf:
                collectjson(c, directory)

    def scss(self, conf=None):
        if conf is None:
            conf = self.conf.get('scss', {})
        if isinstance(conf, dict):
            scss(conf)
        else:
            for c in conf:
                scss(cs)

    def optimize(self, conf=None):
        "Combine and optimize Javascript and CSS files"
        if conf is None:
            conf = self.conf.get('optimize', {})

        optimize(conf, self.indir, self.outdir)

    def appcache(self, conf=None):
        if conf is None:
            conf = self.conf.get('appcache', {})
        appcache(conf, self.indir, self.outdir, self.version)

if __name__ == "__main__":
    if len(sys.argv) > 1:
        args = sys.argv[1:]
    else:
        args = []
    builder = Builder(*args)
    builder.build()



import os
import subprocess
import random
import json

from .collect import readfiles


def optimize(conf, indir, outdir):
    bfile = "rjsconf%s" % (random.random() * 10000)
    bjs = open(bfile, 'w')
    json.dump(conf, bjs)
    bjs.close()

    # Defer to r.js for actual processing
    print '#' * 20
    print "Optimizing with r.js"
    rjs = os.path.dirname(__file__) + "/r.js"
    subprocess.call(["node", rjs, "-o", bfile])
    os.remove(bfile)
    os.remove(outdir + '/' + bfile)
    print "Optimization complete"
    print '#' * 20


def scss(conf):
    import scss
    import logging
    compiler = scss.Scss(scss_opts={'compress': 0})
    logging.getLogger("scss").addHandler(logging.StreamHandler())

    def compile(path, source):
        css = compiler.compile(source)
        outfile = open(path, 'w')
        outfile.write(css)
        outfile.close()

    if 'indir' in conf and 'outdir' in conf:
        files = readfiles(conf['indir'], "scss")
        scss.config.LOAD_PATHS = conf['indir']
        for name, source in files.iteritems():
            if isinstance(source, dict):
                continue
            path = "%s/%s.css" % (conf['outdir'], name)
            compile(path, source)
            print "%s compiled from %s/%s.scss" % (path, conf['indir'], name)



import os


def init(conf, indir):
    jsdir = conf.get('js', indir + '/js')
    cssdir = conf.get('css', indir + '/css')
    assets = os.path.dirname(os.path.dirname(__file__))
    if not os.path.exists(jsdir + '/wq'):
        os.symlink(assets + '/js', jsdir + '/wq')
    if not os.path.exists(cssdir + '/wq'):
        os.symlink(assets + '/css', cssdir + '/wq')



import os
from setuptools import setup, find_packages

LONG_DESCRIPION = """
JavaScript web apps with RequireJS, jQuery Mobile, Mustache, and Leaflet
"""

def long_description():
    """Return long description from README.rst if it's present
    because it doesn't get installed."""
    try:
        return open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
    except IOError:
        return LONG_DESCRIPTION

def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}


package_data = get_package_data('wq')
package_data['wq'].append('app/util/r.js')

setup(
    name = 'wq.app',
    version = '0.4.1',
    author='S. Andrew Sheppard',
    author_email='andrew@wq.io',
    url='http://wq.io/wq.app',
    license='MIT',
    packages=find_packages(),
    package_data=package_data,
    namespace_packages=['wq'],
    description='JavaScript web apps with RequireJS, jQuery Mobile, Mustache, and Leaflet',
    long_description=long_description(),
    install_requires=['pyscss'],
    scripts=['wq/app/util/wq'],
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: JavaScript',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Text Processing :: Markup :: HTML'
    ]
)
